@@ -31,6 +31,7 @@ import json
 import urllib3
 import requests
 
+import yaml
 import micropipenv
 from termcolor import colored
 from yaspin import yaspin
@@ -38,6 +39,7 @@ from yaspin.spinners import Spinners
 from invectio import gather_library_usage
 from thoth.analyzer import run_command
 from thoth.python import Project
+from thoth.python import Constraints
 from thoth.common import ThothAdviserIntegrationEnum
 from thoth.common import get_justification_link as jl
 from thoth.common import cwd
@@ -49,6 +51,7 @@ from .swagger_client import BuildAnalysisApi
 from .swagger_client import Configuration
 from .swagger_client import PythonStack
 from .swagger_client import AdviseInput
+from .swagger_client import ProvenanceInput
 from .swagger_client import AdviseApi
 from .swagger_client import ImageAnalysisApi
 from .swagger_client import ProvenanceApi
@@ -73,6 +76,7 @@ _LOGGER = logging.getLogger(__name__)
 _RETRY_ON_ERROR_COUNT = int(os.getenv("THAMOS_RETRY_ON_ERROR_COUNT", 3))
 _RETRY_ON_ERROR_SLEEP = float(os.getenv("THAMOS_RETRY_ON_ERROR_SLEEP", 3.0))
 _THAMOS_TIMEOUT = int(os.getenv("THAMOS_TIMEOUT", 2000))
+_THAMOS_TOKEN = os.getenv("THAMOS_TOKEN")
 _SOURCE = {
     "adviser": "advise/python",
     "provenance-checker": "provenance/python",
@@ -109,7 +113,7 @@ def with_api_client(func: typing.Callable):
     return wrapper
 
 
-def is_analysis_ready(analysis_id: str) -> bool:
+def is_analysis_ready(analysis_id: str, *, verify_tls: bool = True) -> bool:
     """Handle the the multiple response types available while asking for result of a anaylsis result."""
     config = Configuration()
     host = thoth_config.explicit_host
@@ -118,7 +122,9 @@ def is_analysis_ready(analysis_id: str) -> bool:
         host = thoth_config.content.get("host") or config.host
     source = analysis_id.rsplit("-", maxsplit=2)[0]
     source_url = _SOURCE.get(source)
-    response = requests.get(f"https://{host}/api/v1/{source_url}/{analysis_id}")
+    response = requests.get(
+        f"https://{host}/api/v1/{source_url}/{analysis_id}", verify=verify_tls
+    )
     if response.status_code == 202:
         return False
     elif response.status_code in (200, 400):
@@ -131,7 +137,13 @@ def is_analysis_ready(analysis_id: str) -> bool:
     )
 
 
-def _wait_for_analysis(status_func: Callable[..., Any], analysis_id: str, timeout: typing.Optional[int] = None) -> None:
+def _wait_for_analysis(
+    status_func: Callable[..., Any],
+    analysis_id: str,
+    *,
+    timeout: typing.Optional[int] = None,
+    verify_tls: bool = True,
+) -> None:
     """Wait for ongoing analysis to finish."""
     # noqa
     @contextmanager
@@ -158,10 +170,11 @@ def _wait_for_analysis(status_func: Callable[..., Any], analysis_id: str, timeou
             if timeout and monotonic() - start_time > timeout:
                 raise TimeoutError(
                     f"Thoth backend did not respond in time, timeout set "
-                    f"to {_THAMOS_TIMEOUT} - see {jl('thamos_timeout')}"
+                    f"to {timeout} - see {jl('thamos_timeout')}"
                 )
+
             try:
-                response = status_func(analysis_id)
+                response = status_func(analysis_id, verify_tls=verify_tls)
             except Exception as exc:
                 if retries >= _RETRY_ON_ERROR_COUNT:
                     raise
@@ -310,9 +323,10 @@ def _get_origin() -> typing.Optional[str]:
 def advise_using_config(
     pipfile: str,
     pipfile_lock: str,
-    config: str = None,
+    config: str,
     *,
     runtime_environment_name: typing.Optional[str] = None,
+    constraints: typing.Optional[str] = None,
     src_path: typing.Optional[str] = None,
     recommendation_type: str = None,
     dev: bool = False,
@@ -340,6 +354,7 @@ def advise_using_config(
     return advise(
         pipfile=pipfile,
         pipfile_lock=pipfile_lock,
+        constraints=constraints,
         recommendation_type=recommendation_type,
         src_path=src_path if src_path is not None else thoth_config.config_path,
         runtime_environment=thoth_config.get_runtime_environment(
@@ -355,12 +370,13 @@ def advise_using_config(
         count=count,
         debug=debug,
         origin=origin,
-        timout=timeout,
+        timeout=timeout,
         github_event_type=github_event_type,
         github_check_run_id=github_check_run_id,
         github_installation_id=github_installation_id,
         github_base_repo_url=github_base_repo_url,
         source_type=source_type,
+        verify_tls=thoth_config.tls_verify,
     )
 
 
@@ -369,17 +385,18 @@ def advise(
     api_client: ApiClient,
     pipfile: str,
     pipfile_lock: str,
-    recommendation_type: str = None,
     *,
-    runtime_environment: dict = None,
+    constraints: typing.Optional[str] = None,
+    recommendation_type: typing.Optional[str] = None,
+    runtime_environment: typing.Optional[dict] = None,
     src_path: str = ".",
-    runtime_environment_name: str = None,
+    runtime_environment_name: typing.Optional[str] = None,
     dev: bool = False,
     no_static_analysis: bool = False,
     no_user_stack: bool = False,
     nowait: bool = False,
     force: bool = False,
-    limit: int = None,
+    limit: typing.Optional[int] = None,
     count: int = 1,
     debug: bool = False,
     origin: typing.Optional[str] = None,
@@ -389,15 +406,18 @@ def advise(
     github_installation_id: typing.Optional[int] = None,
     github_base_repo_url: typing.Optional[str] = None,
     source_type: typing.Optional[ThothAdviserIntegrationEnum] = None,
+    justification: typing.Optional[Dict] = None,
+    stack_info: typing.Optional[Dict] = None,
     kebechet_metadata: typing.Optional[Dict] = None,
+    verify_tls: bool = True,
 ) -> typing.Optional[tuple]:
     """Submit a stack for adviser checks and wait for results."""
     if not pipfile:
         raise ValueError("No Pipfile content provided for advises")
 
     if runtime_environment is None:
-        runtime_environment = (
-            thoth_config.get_runtime_environment(runtime_environment_name) or dict()
+        runtime_environment = thoth_config.get_runtime_environment(
+            runtime_environment_name
         )
 
     if runtime_environment_name is None and runtime_environment:
@@ -438,9 +458,22 @@ def advise(
         # Override recommendation type specified explicitly in the runtime environment entry.
         runtime_environment.pop("recommendation_type", None)
 
-    advise_input = AdviseInput(
-        stack, runtime_environment=runtime_environment, library_usage=library_usage
-    )
+    input_args: Dict[str, Any] = {
+        "application_stack": stack,
+        "runtime_environment": runtime_environment,
+        "library_usage": library_usage,
+    }
+
+    if kebechet_metadata:
+        input_args["kebechet_metadata"] = kebechet_metadata
+    if justification:
+        input_args["justification"] = justification
+    if stack_info:
+        input_args["stack_info"] = stack_info
+    if constraints:
+        input_args["constraints"] = constraints
+
+    advise_input = AdviseInput(**input_args)
     api_instance = AdviseApi(api_client)
 
     if recommendation_type:
@@ -479,8 +512,8 @@ def advise(
     if github_base_repo_url is not None:
         parameters["github_base_repo_url"] = github_base_repo_url
 
-    if kebechet_metadata is not None:
-        parameters["kebechet_metadata"] = kebechet_metadata
+    if _THAMOS_TOKEN:
+        parameters["token"] = _THAMOS_TOKEN
 
     response = api_instance.post_advise_python(advise_input, **parameters)
 
@@ -498,7 +531,9 @@ def advise(
     if nowait:
         return response.analysis_id
     # We call custom status function for advise until swagger client supports mulitple response codes.
-    _wait_for_analysis(is_analysis_ready, response.analysis_id, timeout)
+    _wait_for_analysis(
+        is_analysis_ready, response.analysis_id, timeout=timeout, verify_tls=verify_tls
+    )
     _LOGGER.debug("Retrieving adviser result for %r", response.analysis_id)
     response = _retrieve_analysis_result(
         api_instance.get_advise_python, response.analysis_id
@@ -514,7 +549,7 @@ def advise(
 def advise_here(
     recommendation_type: typing.Optional[str] = None,
     *,
-    runtime_environment: dict = None,
+    runtime_environment: typing.Optional[dict] = None,
     src_path: str = ".",
     runtime_environment_name: typing.Optional[str] = None,
     dev: bool = False,
@@ -532,7 +567,10 @@ def advise_here(
     github_installation_id: typing.Optional[int] = None,
     github_base_repo_url: typing.Optional[str] = None,
     source_type: typing.Optional[ThothAdviserIntegrationEnum] = None,
+    justification: typing.Optional[Dict] = None,
+    stack_info: typing.Optional[Dict] = None,
     kebechet_metadata: typing.Optional[Dict] = None,
+    verify_tls: bool = True,
 ) -> typing.Optional[tuple]:
     """Run advise in current directory, requires no arguments."""
     requirements_format = thoth_config.requirements_format
@@ -549,7 +587,9 @@ def advise_here(
             )
 
         project = Project.from_files(
-            without_pipfile_lock=not os.path.exists("Pipfile.lock")
+            pipfile_path="Pipfile",
+            pipfile_lock_path="Pipfile.lock",
+            without_pipfile_lock=not os.path.exists("Pipfile.lock"),
         )
 
         if (
@@ -574,12 +614,22 @@ def advise_here(
             f"Unknown configuration option for requirements format: {requirements_format!r}"
         )
 
+    constraints_str = None
+    if os.path.exists("constraints.txt"):
+        _LOGGER.info("Using constraints.txt file located in %r", os.getcwd())
+        with open("constraints.txt") as constraints_file:
+            constraints_str = constraints_file.read()
+
+        # Try to load constraints before the request to verify their correctness without sending request to the backend.
+        Constraints.from_string(constraints_str)
+
     pipfile = project.pipfile.to_string()
     pipfile_lock_str = project.pipfile_lock.to_string() if project.pipfile_lock else ""
 
     return advise(
         pipfile=pipfile,
         pipfile_lock=pipfile_lock_str,
+        constraints=constraints_str,
         recommendation_type=recommendation_type,
         src_path=src_path,
         runtime_environment=runtime_environment,
@@ -599,7 +649,10 @@ def advise_here(
         github_check_run_id=github_check_run_id,
         github_installation_id=github_installation_id,
         github_base_repo_url=github_base_repo_url,
+        justification=justification,
+        stack_info=stack_info,
         kebechet_metadata=kebechet_metadata,
+        verify_tls=verify_tls,
     )
 
 
@@ -614,15 +667,41 @@ def provenance_check(
     debug: bool = False,
     origin: typing.Optional[str] = None,
     timeout: typing.Optional[int] = None,
+    justification: typing.Optional[Dict[str, Any]] = None,
+    stack_info: typing.Optional[Dict[str, Any]] = None,
+    kebechet_metadata: typing.Optional[Dict[str, Any]] = None,
+    verify_tls: bool = False,
 ) -> typing.Optional[tuple]:
     """Submit a stack for provenance checks and wait for results."""
     if not pipfile:
         raise ValueError("No Pipfile content provided for provenance checks")
 
-    stack = PythonStack(requirements=pipfile, requirements_lock=pipfile_lock)
+    input_args: Dict[str, Any] = {
+        "application_stack": PythonStack(
+            requirements=pipfile, requirements_lock=pipfile_lock
+        ),
+    }
+    if justification:
+        input_args["justification"] = justification
+    if stack_info:
+        input_args["stack_info"] = stack_info
+    if kebechet_metadata:
+        input_args["kebechet_metadata"] = kebechet_metadata
+
+    provenance_input = ProvenanceInput(**input_args)
+
+    provenance_kwargs: Dict[str, Any] = {
+        "debug": debug,
+        "force": force,
+    }
+    if origin:
+        provenance_kwargs["origin"] = origin
+    if _THAMOS_TOKEN:
+        provenance_kwargs["token"] = _THAMOS_TOKEN
+
     api_instance = ProvenanceApi(api_client)
     response = api_instance.post_provenance_python(
-        stack, debug=debug, force=force, origin=origin
+        provenance_input, **provenance_kwargs
     )
     _LOGGER.info(
         "Successfully submitted provenance check analysis %r to %r",
@@ -635,7 +714,9 @@ def provenance_check(
     if nowait:
         return response.analysis_id
 
-    _wait_for_analysis(is_analysis_ready, response.analysis_id, timeout)
+    _wait_for_analysis(
+        is_analysis_ready, response.analysis_id, timeout=timeout, verify_tls=verify_tls
+    )
     _LOGGER.debug("Retrieving provenance check result for %r", response.analysis_id)
     response = _retrieve_analysis_result(
         api_instance.get_provenance_python, response.analysis_id
@@ -654,6 +735,10 @@ def provenance_check_here(
     debug: bool = False,
     origin: typing.Optional[str] = None,
     timeout: typing.Optional[int] = None,
+    justification: typing.Optional[Dict[str, Any]] = None,
+    stack_info: typing.Optional[Dict[str, Any]] = None,
+    kebechet_metadata: typing.Optional[Dict[str, Any]] = None,
+    verify_tls: bool = True,
 ) -> typing.Optional[tuple]:
     """Submit a provenance check in current directory."""
     if not os.path.isfile("Pipfile"):
@@ -671,6 +756,10 @@ def provenance_check_here(
             debug=debug,
             origin=origin,
             timeout=timeout,
+            justification=justification,
+            stack_info=stack_info,
+            kebechet_metadata=kebechet_metadata,
+            verify_tls=verify_tls,
         )
 
 
@@ -718,7 +807,9 @@ def image_analysis(
     if nowait:
         return response.analysis_id
 
-    _wait_for_analysis(is_analysis_ready, response.analysis_id, timeout)
+    _wait_for_analysis(
+        is_analysis_ready, response.analysis_id, timeout=timeout, verify_tls=verify_tls
+    )
     _LOGGER.debug(
         "Retrieving image analysis result result for %r", response.analysis_id
     )
@@ -972,9 +1063,14 @@ def install(
                 )
 
         if runtime_environment_name is None:
-            config_entry = thoth_config.get_runtime_environment(runtime_environment_name)
+            config_entry = thoth_config.get_runtime_environment(
+                runtime_environment_name
+            )
             runtime_environment_name = config_entry["name"]
-        _LOGGER.info("Installing requirements for runtime environment %r", runtime_environment_name)
+        _LOGGER.info(
+            "Installing requirements for runtime environment %r",
+            runtime_environment_name,
+        )
 
         _LOGGER.info(
             "Using %r installation method to install dependencies stated in %r",
@@ -1038,3 +1134,109 @@ def list_python_package_indexes(api_client: ApiClient) -> typing.Dict[str, Any]:
     return (
         PythonPackagesApi(api_client).list_python_package_indexes().to_dict()["indexes"]
     )
+
+
+def write_files(
+    requirements: typing.Dict[str, Any],
+    requirements_lock: typing.Dict[str, Any],
+    requirements_format: str,
+) -> None:
+    """Write content of Pipfile/Pipfile.lock or requirements.in/txt to the current directory."""
+    project = Project.from_dict(requirements, requirements_lock)
+    if requirements_format == "pipenv":
+        _LOGGER.debug("Writing to Pipfile/Pipfile.lock in %r", os.getcwd())
+        project.to_files(keep_thoth_section=True)
+    elif requirements_format in ("pip", "pip-tools", "pip-compile"):
+        _LOGGER.debug("Writing to requirements.in/requirements.txt in %r", os.getcwd())
+        project.to_pip_compile_files()
+        _LOGGER.debug("No changes to Pipfile to write")
+    else:
+        raise ValueError(
+            f"Unknown requirements format, supported are 'pipenv' and 'pip': {requirements_format!r}"
+        )
+
+
+def load_files(requirements_format: str) -> typing.Tuple[str, typing.Optional[str]]:
+    """Load Pipfile/Pipfile.lock or requirements.in/txt from the current directory."""
+    if requirements_format == "pipenv":
+        _LOGGER.info("Using Pipenv files located in %r directory", os.getcwd())
+        pipfile_lock_exists = os.path.exists("Pipfile.lock")
+
+        if pipfile_lock_exists:
+            _LOGGER.info(
+                "Submitting Pipfile.lock as a base for user's stack scoring - see %s",
+                jl("user_stack"),
+            )
+
+        project = Project.from_files(
+            without_pipfile_lock=not os.path.exists("Pipfile.lock")
+        )
+
+        if (
+            pipfile_lock_exists
+            and project.pipfile_lock.meta.hash["sha256"]
+            != project.pipfile.hash()["sha256"]
+        ):
+            _LOGGER.error(
+                "Pipfile hash stated in Pipfile.lock %r does not correspond to Pipfile hash %r - was Pipfile "
+                "adjusted? This error is not critical.",
+                project.pipfile_lock.meta.hash["sha256"][:6],
+                project.pipfile.hash()["sha256"][:6],
+            )
+    elif requirements_format in ("pip", "pip-tools", "pip-compile"):
+        _LOGGER.info("Using requirements.txt file located in %r directory", os.getcwd())
+        project = Project.from_pip_compile_files(allow_without_lock=True)
+    else:
+        raise ValueError(
+            f"Unknown configuration option for requirements format: {requirements_format!r}"
+        )
+    return (
+        project.pipfile.to_string(),
+        project.pipfile_lock.to_string() if project.pipfile_lock else None,
+    )
+
+
+def write_configuration(
+    advised_configuration: dict,
+    recommendation_type: str = None,
+    dev: bool = False,
+) -> None:
+    """Create thoth configuration file."""
+    if not advised_configuration:
+        _LOGGER.debug("No advises on configuration, nothing to adjust")
+        return
+
+    if "name" not in advised_configuration:
+        _LOGGER.error(
+            "Cannot adjust Thoth's configuration based on advises: No name found in Thoth's configuration entry"
+        )
+        return
+
+    _LOGGER.debug("Reading Thoth's configuration file")
+    with open(".thoth.yaml", "r") as thoth_yaml_file:
+        content = yaml.safe_load(thoth_yaml_file.read())
+
+    for idx, runtime_environment_entry in enumerate(
+        content.get("runtime_environments", [])
+    ):
+        if runtime_environment_entry.get("name") == advised_configuration["name"]:
+            _LOGGER.debug(
+                "Adjusting configuration entry for %r based on recommendations",
+                advised_configuration["name"],
+            )
+            runtime_environment_entry = advised_configuration
+            if recommendation_type:
+                runtime_environment_entry["recommendation_type"] = recommendation_type
+            runtime_environment_entry["dev"] = dev
+            content["runtime_environments"][idx] = runtime_environment_entry
+            break
+    else:
+        _LOGGER.error(
+            "Cannot adjust Thoth's configuration based on advises: No runtime environment entry with name %r found",
+            advised_configuration["name"],
+        )
+        return
+
+    _LOGGER.debug("Writing adjusted Thoth's configuration file")
+    with open(".thoth.yaml", "w") as thoth_yaml_file:
+        yaml.safe_dump(content, thoth_yaml_file)
